#include "JunctionBoxes.h"

#define BOX_CONNECTIVITY 0
#define BOX_EXISTS 1
#define BOX_HEIGHT_VAR 2

#define MIN_BOX_WIDTH 0.3f

#define CONNECTOR_WIDTH (GRID_SIZE * 0.1f)

JunctionBoxes::JunctionBoxes(int typeID) : Generator(typeID)
{
}

JunctionBoxes::~JunctionBoxes(void)
{
}

void JunctionBoxes::SetVariables()
{
	m_layerVariables[BOX_CONNECTIVITY].Initialize(this, "Connectivity", 0.0f, 1.1, 0.0f);
	m_layerVariables[BOX_EXISTS].Initialize(this, "Node exists", 0.5, 1.0f, 0);
	m_layerVariables[BOX_HEIGHT_VAR].Initialize(this, "Node height", MIN_BOX_WIDTH, 2.0f, MIN_BOX_WIDTH);

	m_variableCount = 3;
}

void JunctionBoxes::BuildCollection(Entity3dCollection *collection, WindowMapping &mapping)
{
	m_layerVariables[BOX_EXISTS].PrepareDownSample(mapping);
	m_layerVariables[BOX_CONNECTIVITY].PrepareDownSample(mapping);
	m_layerVariables[BOX_HEIGHT_VAR].PrepareDownSample(mapping);

//	LOG("Doing again!");

	int size_x = mapping.GetPhysicalWidth(), size_y = mapping.GetPhysicalHeight();

	//iterating over space
	for(int j = 0; j < size_y; j++)
	{
		for(int i = 0; i < size_x; i++)	
		{
			float xpos = i, ypos = j;
			mapping.GetWorldCoordinates(xpos, ypos);
			//LOG("Translating (%d, %d) to (%g, %g)", i, j, xpos, ypos);

			float boxExists = m_layerVariables[BOX_EXISTS].GetDownSampledValue(i,j, 0.3f);

			if(boxExists > 0.0f)
			{
				float boxHeight = m_layerVariables[BOX_HEIGHT_VAR].GetDownSampledValue(i,j);
				//draw this cell
				Entity3d ent;
				ent.Initialize(1.0f, 1.0f, boxHeight);
				//int angle = (int)DivFloat(m_layerVariables[BOX_CONNECTIVITY].GetDownSampledValue(i,j), 1.0f);
				float value = (m_layerVariables[BOX_CONNECTIVITY].GetDownSampledValue(i,j, 0.5f));

				if (value > 1.0f) value = 1.0; //CLARMP
				if (value < 0.0f) value = 0.0; //CLARMP

				unsigned char connections = (unsigned char) 0xF * value;
				ent.Connectivity.ConnectionsValue = connections;
				collection->AddEntity(ent, i, j, mapping);
			}
		}
	}

	//afterwards...
	collection->EnsureAdjacentConnectivity();
}
